%% Microstructure simulation: hexagonal prisms in periodic RVE -> ANSYS input file
% by Vaclav Nezerka 10/08/2013

clc; close all; clear all
tic
display('---------------------------------')
display('Running the microstructure generation input file for ANSYS')

%% Input

cAgg = 0.08; % volume fraction of aggregates in the mix
numberOfCrystals = 20;
particleRadius = 1.5e-3; % radius of circumscribed circle around the modeled hexagonal prisms.
particleHeight = 1e-3; % height of the modeled prisms

sphereRadius = sqrt((particleHeight/2)^2+particleRadius^2); % radius of circumscribed sphere to check colisions
cubeSizeRVE = (numberOfCrystals*1.5*sqrt(3)*particleRadius^2*particleHeight/cAgg)^(1/3);
minDistBetweenGrains = 4e-4;    % minimum distance between grains

figureOut = true; % true = plots the generated microstructure, false = no plotting
coordinatesOfVertiecesOut = true; % true = prints the coordinates of individual vertieces out

%% ANSYS and meshing options

controlBoundaryMesh = true;         % whether to control the elementsize on the boundary
controlMeshSizeInMatrix = true;         % whether to control the elementsize manually
    elSize = particleRadius/3.3;          % element size 
    elSizeBoundary = particleRadius/3.3;  % element size at the boundary of the sample
    elSizeGrain = particleRadius/3.3;     % element size inside grains
 
%% Calculations: placement of grains

particlesOrientations = f_crystalsGeneration3D(cAgg, cubeSizeRVE^3, particleRadius, particleHeight);
fprintf('%d orientations were generated, placement of crystals...\n', size(particlesOrientations,1))

placedParticles = f_placeCrystals(particlesOrientations,cubeSizeRVE,minDistBetweenGrains,particleRadius,particleHeight,sphereRadius,coordinatesOfVertiecesOut,cAgg);
display('100% crystals placed, printing ANSYS input file...')
nParticles = length(placedParticles);

%% preparation of ANSYS input file generation

ansysInputFile = ['outputMicrostructure/ansysInput/',num2str(size(particlesOrientations,1)),'_crystals_c_',num2str(cAgg),'.in'];
sampleDim = [cubeSizeRVE; cubeSizeRVE; cubeSizeRVE];
negTol = 1e-7; % negligible tolerance dimension, used for BTOL operation
f = fopen(ansysInputFile,'w');

%% header of ANSYS input file

fprintf(f,'!Input file for ANSYS prepared by Venca (VN)\n');
fprintf(f,'!The file defines generation of mesh in ANSYS to be used for FEM analysis\n');
fprintf(f,'! \n');
fprintf(f,'!Date: %s\n', datestr(now));
fprintf(f,'!----------------------------------------------------------------------\n');
fprintf(f,'\n');
fprintf(f,'fini\n/CLEAR,start\n'); % starts new file
%fprintf(f,'/PREP7\n\n'); % starts preprocessor
fprintf(f,'/PREP7\nbtol,%d\n\n',negTol); % starts preprocessor, tolerance for boolean operations

if figureOut
    cubeEdges = [0,0,0; cubeSizeRVE,0,0;
        cubeSizeRVE,0,0; cubeSizeRVE,cubeSizeRVE,0;
        cubeSizeRVE,cubeSizeRVE,0; 0,cubeSizeRVE,0;
        0,cubeSizeRVE,0; 0,0,0;
        0,0,cubeSizeRVE; cubeSizeRVE,0,cubeSizeRVE;
        cubeSizeRVE,0,cubeSizeRVE; cubeSizeRVE,cubeSizeRVE,cubeSizeRVE;
        cubeSizeRVE,cubeSizeRVE,cubeSizeRVE; 0,cubeSizeRVE,cubeSizeRVE;
        0,cubeSizeRVE,cubeSizeRVE; 0,0,cubeSizeRVE;
        0,0,0; 0,0,cubeSizeRVE;
        cubeSizeRVE,0,0; cubeSizeRVE,0,cubeSizeRVE;
        cubeSizeRVE,cubeSizeRVE,0; cubeSizeRVE,cubeSizeRVE,cubeSizeRVE;
        0,cubeSizeRVE,0; 0,cubeSizeRVE,cubeSizeRVE];
    
    for ce = 1:12
        plot3([cubeEdges(2*ce-1,1),cubeEdges(2*ce,1)],[cubeEdges(2*ce-1,2),cubeEdges(2*ce,2)],[cubeEdges(2*ce-1,3),cubeEdges(2*ce,3)])
        hold on
    end
end

%% MATERIALS

fprintf(f,'!----------------------------------------------------------------------\n');
fprintf(f,'!DEFINITION OF MATERIALS\n');
fprintf(f,'!----------------------------------------------------------------------\n\n');

fprintf(f,'MP,EX,1,%g\n', 80);     % grains; MP = material properties; EX,1,80 = modulus of elasticity equal to 80 and material reference number is 1
fprintf(f,'MP,PRXY,1,%g\n', 0.2);  % Poisson's ratio equal to 0.2

fprintf(f,'MP,EX,2,%g\n', 30);     % matrix
fprintf(f,'MP,PRXY,2,%g\n', 0.2);

fprintf(f,'MP,EX,3,%g\n', 10);     % interface
fprintf(f,'MP,PRXY,3,%g\n\n', 0.2);

%% MESH PROPERTIES

fprintf(f,'!----------------------------------------------------------------------\n');
fprintf(f,'!MESH PROPERTIES, ELEMENT TYPES\n');
fprintf(f,'!----------------------------------------------------------------------\n\n');

fprintf(f,'ET,1,MESH200,2\n'); % 3D line, auxiliary, no integration points, just to create a raster
fprintf(f,'ET,2,MESH200,4\n'); % 3D triangle, auxiliary, no integration points, just to create a raster
fprintf(f,'ET,3,SOLID72\n');   % for particles
fprintf(f,'ET,4,SOLID72\n');   % for matrix
fprintf(f,'ET,5,INTER205\n');  % contacts

fprintf(f,'MSHAPE,1,3D\n\n');  % 0 - hexahedral-shaped elements , 1 - tetrahedral-shaped elements

%% Generation of particles

fprintf(f,'!----------------------------------------------------------------------\n');
fprintf(f,'!CRYSTALS\n');
fprintf(f,'!----------------------------------------------------------------------\n\n');

keyPointCounter = 0;
for j = 1:nParticles % individual grains
    
    fprintf(f,'! ---> Crystal number %d <---\n\n',j);
    nAreas = length(placedParticles(j).areasOnCrystal);
    keyPointCentroidCounter = keyPointCounter+1;
    keyPointCounter = keyPointCounter+1;
    actualCentroid = placedParticles(j).areasOnCrystal.centroid;
    fprintf(f,'K,%d,%.10e,%.10e,%.10e',keyPointCentroidCounter,actualCentroid(1),actualCentroid(2),actualCentroid(3));
    fprintf(f,'\n');
    
    for k = 1:nAreas
        actualArea = placedParticles(j).areasOnCrystal(k).area;
        nKeypoints = size(placedParticles(j).areasOnCrystal(k).area,1);
        
        keyPointCounter = keyPointCounter+10;
        for l = 1:nKeypoints
            keyPointCounter = keyPointCounter+1;
            actualKeypoint = placedParticles(j).areasOnCrystal(k).area(l,:);
            fprintf(f,'K,%d,%.10e,%.10e,%.10e',keyPointCounter,actualKeypoint(1),actualKeypoint(2),actualKeypoint(3));
            fprintf(f,'\n');
        end
        
        subvolumeCounter = 0;
        nSubvolumes = nKeypoints-2;

        for l = 1:nSubvolumes
            subvolumeCounter = subvolumeCounter+1;
            fprintf(f,'V,%d',keyPointCentroidCounter);
            
            if l == 1
                for m = 1:3
                    fprintf(f,',%d',keyPointCounter-nKeypoints+m+subvolumeCounter-1);
                    if m == 1
                        firstKeypointInSubvolume = keyPointCounter-nKeypoints+m+subvolumeCounter-1;
                        firstKeypointInArea = firstKeypointInSubvolume;
                    end
                end
                fprintf(f,'\n');

            else
                for m = 1:3
                    if m == 1
                        fprintf(f,',%d',firstKeypointInSubvolume);
                        firstKeypointInSubvolume = keyPointCounter-nKeypoints+m+subvolumeCounter-1;
                    else
                        fprintf(f,',%d',keyPointCounter-nKeypoints+m+subvolumeCounter-1);
                        lastKeypointInArea = keyPointCounter-nKeypoints+m+subvolumeCounter-1;
                    end
                end
                fprintf(f,'\n');
            end
        end
        
        if nKeypoints > 4 && l ~= 1
            for l = 1:nSubvolumes
                fprintf(f,'V,%d,%d,%d,%d',keyPointCentroidCounter,lastKeypointInArea,firstKeypointInArea,firstKeypointInArea+l);
                fprintf(f,'\n');
            end
        end
        
        if figureOut
            p = patch(actualArea(:,1),actualArea(:,2),actualArea(:,3),[0.7,0.7,1.0]);
            hold on
        end
    end
    
    fprintf(f,'\n');
end
fprintf(f,'\n');
fprintf(f,'VSEL,S,MAT,,0,0\n');
fprintf(f,'VADD,ALL\n'); % boolean add of areas (first triangle and the current one are merged into a single shape)
fprintf(f,'NUMCMP,VOLUME\n');  % compress volume numbers (renumbers the items and deletes unused ones)

fprintf(f,'\n');
fprintf(f,'VSEL,S,MAT,,0,0\n'); % select volumes (s = new set, MAT = material number associated with the area, 
                                % no component needed if MAT is selected (,,), minimum number of selected areas = 0 
                                % and last 0 stands for selection of volumes only (without lines, nodes...)
fprintf(f,'VATT,1\n'); % associate element attributes with the selected unmeshed volumes, material number 1

%% Matrix definition for ANSYS

fprintf(f,'!----------------------------------------------------------------------\n');
fprintf(f,'!MATRIX\n');
fprintf(f,'!----------------------------------------------------------------------\n\n');

fprintf(f,'BLOCK,%.10e,%.10e,%.10e,%.10e,%.10e,%.10e\n',0,sampleDim(1),0,sampleDim(2),0,sampleDim(3));  % creates a block (x1, x2, y1, y2, z1, z2)
fprintf(f,'VSEL,S,MAT,,0,0\n'); % select volumes (S = new set, MAT = material number associated with the area,
% no component needed if MAT is selected (,,), minimum number of selected areas = 0
% and last 0 stands for selection of areas only (without lines, nodes...)
fprintf(f, 'VATT,2\n'); % associate element attributes with the selected unmeshed areas, material number 2 (matrix)
fprintf(f, 'NUMCMP,VOLUME\n\n'); % compress area numbers (renumbers the items and deletes unused ones)

% Boolean subtraction of aggregates from the matrix
boolLimit = 1e6; % if boolLimit > 50 --> unstable boolean subtraction in ANSYS
for nFields = 1:ceil(nParticles/boolLimit)
    fprintf(f, 'VSEL,S,,,%d,%d\n', (nFields-1)*boolLimit+1,min(nFields*boolLimit,nParticles)); % select volumes (s = new set, per 20 pcs)
    fprintf(f, 'VSEL,A,MAT,,2,2\n'); % select volumes (A = additional set, material 2 = matrix)
    fprintf(f, 'VOVLAP,ALL\n'); % overlaps all volumes
    fprintf(f, 'VSEL,S,MAT,,0,0\n'); % select volumes (S = new set, 0 = min number of areas, 0 = no nodes, lines...)
    fprintf(f, 'VATT,2\n'); % associate element attributes with the selected unmeshed volumes, material number 2 (matrix)
    fprintf(f, 'NUMCMP, ALL\n\n'); % compress all numbers (renumbers the items and deletes unused ones)
end

%% ASSIGNMENT OF NAMES TO EDGES AND AREAS TO GENERATE PERIODIC MESH

fprintf(f,'!Names of exterior areas\n');
fprintf(f,'CSYS,0\n');
% A1
fprintf(f,'VSEL,ALL\nASLV,S\n');
fprintf(f,'ASEL,R,LOC,Z,%.10e,%.10e\n',0-negTol,0+negTol); % chooses an area
fprintf(f,'CM,P1,AREA\n');   % assigns a name to the area
% A2
fprintf(f,'VSEL,ALL\nASLV,S\n');
fprintf(f,'ASEL,R,LOC,Z,%.10e,%.10e\n',cubeSizeRVE-negTol,cubeSizeRVE+negTol); % chooses an area
fprintf(f,'CM,P2,AREA\n');   % assigns a name to the area
% A3
fprintf(f,'VSEL,ALL\nASLV,S\n');
fprintf(f,'ASEL,R,LOC,Y,%.10e,%.10e\n',0-negTol,0+negTol); % chooses an area
fprintf(f,'CM,P3,AREA\n');   % assigns a name to the area
% A4
fprintf(f,'VSEL,ALL\nASLV,S\n');
fprintf(f,'ASEL,R,LOC,X,%.10e,%.10e\n',cubeSizeRVE-negTol,cubeSizeRVE+negTol); % chooses an area
fprintf(f,'CM,P4,AREA\n');   % assigns a name to the area
% A5
fprintf(f,'VSEL,ALL\nASLV,S\n');
fprintf(f,'ASEL,R,LOC,Y,%.10e,%.10e\n',cubeSizeRVE-negTol,cubeSizeRVE+negTol); % chooses an area
fprintf(f,'CM,P5,AREA\n');   % assigns a name to the area
% A6
fprintf(f,'VSEL,ALL\nASLV,S\n');
fprintf(f,'ASEL,R,LOC,X,%.10e,%.10e\n',0-negTol,0+negTol); % chooses an area
fprintf(f,'CM,P6,AREA\n\n');   % assigns a name to the area

% Edges
fprintf(f,'!Names of exterior edges\n');
% H1
fprintf(f,'CMSEL,S,P1\nLSLA,S\n'); % Chooses an area and from this the lines contained within
fprintf(f,'LSEL,R,LOC,Y,%.10e,%.10e\n',0-negTol,0+negTol); % chooses an edge
fprintf(f,'LATT,,,1\n');
fprintf(f,'CM,H1,LINE\n');   % Assigns a name to the edge
% H2
fprintf(f,'CMSEL,S,P1\nLSLA,S\n'); % Chooses an area and from this the lines contained within
fprintf(f,'LSEL,R,LOC,X,%.10e,%.10e\n',cubeSizeRVE-negTol,cubeSizeRVE+negTol); % chooses an edge
fprintf(f,'LATT,,,1\n');
fprintf(f,'CM,H2,LINE\n');   % Assigns a name to the edge
% H3
fprintf(f,'CMSEL,S,P1\nLSLA,S\n'); % Chooses an area and from this the lines contained within
fprintf(f,'LSEL,R,LOC,Y,%.10e,%.10e\n',cubeSizeRVE-negTol,cubeSizeRVE+negTol); % chooses an edge
fprintf(f,'LATT,,,1\n');
fprintf(f,'CM,H3,LINE\n');   % Assigns a name to the edge
% H4
fprintf(f,'CMSEL,S,P1\nLSLA,S\n'); % Chooses an area and from this the lines contained within
fprintf(f,'LSEL,R,LOC,X,%.10e,%.10e\n',0-negTol,0+negTol); % chooses an edge
fprintf(f,'LATT,,,1\n');
fprintf(f,'CM,H4,LINE\n');   % Assigns a name to the edge

% H5
fprintf(f,'CMSEL,S,P2\nLSLA,S\n'); % Chooses an area and from this the lines contained within
fprintf(f,'LSEL,R,LOC,Y,%.10e,%.10e\n',0-negTol,0+negTol); % chooses an edge
fprintf(f,'LATT,,,1\n');
fprintf(f,'CM,H5,LINE\n');   % Assigns a name to the edge
% H6
fprintf(f,'CMSEL,S,P2\nLSLA,S\n'); % Chooses an area and from this the lines contained within
fprintf(f,'LSEL,R,LOC,X,%.10e,%.10e\n',cubeSizeRVE-negTol,cubeSizeRVE+negTol); % chooses an edge
fprintf(f,'LATT,,,1\n');
fprintf(f,'CM,H6,LINE\n');   % Assigns a name to the edge
% H7
fprintf(f,'CMSEL,S,P2\nLSLA,S\n'); % Chooses an area and from this the lines contained within
fprintf(f,'LSEL,R,LOC,Y,%.10e,%.10e\n',cubeSizeRVE-negTol,cubeSizeRVE+negTol); % chooses an edge
fprintf(f,'LATT,,,1\n');
fprintf(f,'CM,H7,LINE\n');   % Assigns a name to the edge
% H8
fprintf(f,'CMSEL,S,P2\nLSLA,S\n'); % Chooses an area and from this the lines contained within
fprintf(f,'LSEL,R,LOC,X,%.10e,%.10e\n',0-negTol,0+negTol); % chooses an edge
fprintf(f,'LATT,,,1\n');
fprintf(f,'CM,H8,LINE\n');   % Assigns a name to the edge

% H9
fprintf(f,'CMSEL,S,P3\nLSLA,S\n'); % Chooses an area and from this the lines contained within
fprintf(f,'LSEL,R,LOC,X,%.10e,%.10e\n',0-negTol,0+negTol); % chooses an edge
fprintf(f,'LATT,,,1\n');
fprintf(f,'CM,H9,LINE\n');   % Assigns a name to the edge
% H10
fprintf(f,'CMSEL,S,P3\nLSLA,S\n'); % Chooses an area and from this the lines contained within
fprintf(f,'LSEL,R,LOC,X,%.10e,%.10e\n',cubeSizeRVE-negTol,cubeSizeRVE+negTol); % chooses an edge
fprintf(f,'LATT,,,1\n');
fprintf(f,'CM,H10,LINE\n');   % Assigns a name to the edge
% H11
fprintf(f,'CMSEL,S,P5\nLSLA,S\n'); % Chooses an area and from this the lines contained within
fprintf(f,'LSEL,R,LOC,X,%.10e,%.10e\n',cubeSizeRVE-negTol,cubeSizeRVE+negTol); % chooses an edge
fprintf(f,'LATT,,,1\n');
fprintf(f,'CM,H11,LINE\n');   % Assigns a name to the edge
% H12
fprintf(f,'CMSEL,S,P5\nLSLA,S\n'); % Chooses an area and from this the lines contained within
fprintf(f,'LSEL,R,LOC,X,%.10e,%.10e\n',0-negTol,0+negTol); % chooses an edge
fprintf(f,'LATT,,,1\n');
fprintf(f,'CM,H12,LINE\n\n');   % Assigns a name to the edge
    
%% ELEMENTS

fprintf(f,'!----------------------------------------------------------------------\n');
fprintf(f,'!GROUPING OF ELEMENTS, ASSIGNMENT OF NAMES\n');
fprintf(f,'!----------------------------------------------------------------------\n\n');

% assign elements to group of grains
fprintf(f,'!1.GRAIN\n');
fprintf(f,'VSEL,S,MAT,,1,1\n');
fprintf(f,'VATT,1,,3\n');
fprintf(f,'CM,GRAIN,VOLUME\n\n');

% assign elements to group matrix
fprintf(f,'!2.MATRIX\n');
fprintf(f,'VSEL,S,MAT,,2,2\n');
fprintf(f,'VATT,2,,4\n');
fprintf(f,'CM,MATRIX,VOLUME\n\n');

%% MESHING

fprintf(f,'!----------------------------------------------------------------------\n');
fprintf(f,'!MESHING\n');
fprintf(f,'!----------------------------------------------------------------------\n\n');

fprintf(f,'NUMMRG,KP,%.10e,%.10e, ,LOW\n\n',negTol,negTol);  % merges key points

if controlBoundaryMesh
 
    % element size at boundary
    fprintf(f,'ESIZE,%.10e\n',elSizeBoundary);
    fprintf(f,'SMRTSIZE,6\n\n');
    
    fprintf(f,'!MESHING OF SAMPLE BOUNDARY:\n');
    
    % edge meshing
    fprintf(f,'!meshing - exterior edges\n');
    % H1 = H3 = H5 = H7
    fprintf(f,'CMSEL,S,H1\n');
    fprintf(f,'LMESH,ALL\n');
    fprintf(f,'CMSEL,S,,LINE\n');
    fprintf(f,'MSHCOPY,LINE,H1,H3,0,0,%.10e,0\n',cubeSizeRVE);
    fprintf(f,'MSHCOPY,LINE,H1,H5,0,0,0,%.10e\n',cubeSizeRVE);
    fprintf(f,'MSHCOPY,LINE,H1,H7,0,0,%.10e,%.10e\n',cubeSizeRVE,cubeSizeRVE);
    
    %H2 = H4 = H6 = H8
    fprintf(f,'CMSEL,S,H2\n');
    fprintf(f,'LMESH,ALL\n');
    fprintf(f,'CMSEL,S,,LINE\n');
    fprintf(f,'MSHCOPY,LINE,H2,H4,0,%.10e,0,0\n',-cubeSizeRVE);
    fprintf(f,'MSHCOPY,LINE,H2,H6,0,0,0,%.10e\n',cubeSizeRVE);
    fprintf(f,'MSHCOPY,LINE,H2,H8,0,%.10e,0,%.10e\n',-cubeSizeRVE,cubeSizeRVE);
    
    %H9 = H10 = H11 = H12
    fprintf(f,'CMSEL,S,H9\n');
    fprintf(f,'LMESH,ALL\n');
    fprintf(f,'CMSEL,S,,LINE\n');
    fprintf(f,'MSHCOPY,LINE,H9,H10,0,%.10e,0,0\n',cubeSizeRVE);
    fprintf(f,'MSHCOPY,LINE,H9,H11,0,%.10e,%.10e,0\n',cubeSizeRVE,cubeSizeRVE);
    fprintf(f,'MSHCOPY,LINE,H9,H12,0,0,%.10e,0\n\n',cubeSizeRVE);
    
    % area meshing
    fprintf(f,'!meshing - exterior areas\n');
    % P1 = P2
    fprintf(f,'CMSEL,S,P1\n');
    fprintf(f,'AMESH,ALL\n');
    fprintf(f,'CMSEL,S,,AREA\n');
    fprintf(f,'MSHCOPY,AREA,P1,P2,0,0,0,%.10e\n',cubeSizeRVE);
    
    %P3 = P5
    fprintf(f,'CMSEL,S,P3\n');
    fprintf(f,'AMESH,ALL\n');
    fprintf(f,'CMSEL,S,,AREA\n');
    fprintf(f,'MSHCOPY,AREA,P3,P5,0,0,%.10e,0\n',cubeSizeRVE);
    
    %P4 = P6
    fprintf(f,'CMSEL,S,P4\n');
    fprintf(f,'AMESH,ALL\n');
    fprintf(f,'CMSEL,S,,AREA\n');
    fprintf(f,'MSHCOPY,AREA,P4,P6,0,%.10e,0,0\n\n',-cubeSizeRVE);
    
%     fprintf(f,'VSEL,ALL\n');
%     fprintf(f,'ASLV,S\n'); % select those areas contained in the selected volumes
%     fprintf(f,'ASEL,R,LOC,X,%.10e,%.10e\n',-negTol,negTol); % choose the area in the given location
%     fprintf(f,'AMESH,ALL\n\n');
%     
%     fprintf(f,'VSEL,ALL\n');
%     fprintf(f,'ASLV,S\n'); % select those areas contained in the selected volumes
%     fprintf(f,'ASEL,R,LOC,Y,%.10e,%.10e\n',-negTol,negTol); % choose the area in the given location
%     fprintf(f,'AMESH,ALL\n\n');
%     
%     fprintf(f,'VSEL,ALL\n');
%     fprintf(f,'ASLV,S\n'); % select those areas contained in the selected volumes
%     fprintf(f,'ASEL,R,LOC,Z,%.10e,%.10e\n',-negTol,negTol); % choose the area in the given location
%     fprintf(f,'AMESH,ALL\n\n');
%     
%     fprintf(f,'VSEL,ALL\n');
%     fprintf(f,'ASLV,S\n'); % select those areas contained in the selected volumes
%     fprintf(f,'ASEL,R,LOC,X,%.10e,%.10e\n',sampleDim(1)-negTol,sampleDim(1)+negTol); % choose the area in the given location
%     fprintf(f,'AMESH,ALL\n\n');
%     
%     fprintf(f,'VSEL,ALL\n');
%     fprintf(f,'ASLV,S\n'); % select those areas contained in the selected volumes
%     fprintf(f,'ASEL,R,LOC,Y,%.10e,%.10e\n',sampleDim(2)-negTol,sampleDim(2)+negTol); % choose the area in the given location
%     fprintf(f,'AMESH,ALL\n\n');
%     
%     fprintf(f,'VSEL,ALL\n');
%     fprintf(f,'ASLV,S\n'); % select those areas contained in the selected volumes
%     fprintf(f,'ASEL,R,LOC,Z,%.10e,%.10e\n',sampleDim(3)-negTol,sampleDim(3)+negTol); % choose the area in the given location
%     fprintf(f,'AMESH,ALL\n\n');
end


% element size
fprintf(f,'ESIZE,%.10e\n',elSizeGrain);
fprintf(f,'LSEL,ALL\n');
fprintf(f,'LMESH,ALL\n\n');
% fprintf(f,'ESIZE,%.10e\n',elSizeGrain);
% fprintf(f,'SMRTSIZE,6\n\n');

% meshing of grains
fprintf(f,'!MESHING OF GRAINS:\n');
fprintf(f,'VSEL,S,MAT,,1,1\n');
fprintf(f,'VMESH,ALL\n\n');

if controlMeshSizeInMatrix
    % element size
    fprintf(f,'ESIZE,%.10e\n',elSize);
    fprintf(f,'SMRTSIZE,6\n\n');
end

% meshing of matrix
fprintf(f,'!MESHING OF MATRIX:\n');
fprintf(f,'VSEL,ALL\n');
fprintf(f,'VMESH,ALL\n\n');

% mesh cleaning and arrangement - unwanted is deleted
fprintf(f,'!MESH CLEANING AND ARRANGEMENT:\n');
fprintf( f,'ASEL,ALL\n');
fprintf( f,'ACLE,ALL\n');
fprintf(f,'NUMCMP,ALL\n\n');

%% LOADING AND SUPPORTS

fprintf(f,'!----------------------------------------------------------------------\n');
fprintf(f,'!LOADING AND SUPPORTS\n');
fprintf(f,'!----------------------------------------------------------------------\n\n');

% supporting of nodes in yz-directions
fprintf(f,'LSEL,S,LOC,Z, %e, %e\n',-negTol,negTol); % select all lines in the given location
fprintf(f,'LSEL,R,LOC,Y, %e, %e\n',-negTol,negTol); % reselect from the current set
fprintf(f,'NSLL,S,1\n'); % select all nodes (interior to line and at keypoints) associated with the selected lines
fprintf(f,'D,ALL,UZ,0.0\n');
fprintf(f,'D,ALL,UY,0.0\n');

% supporting of nodes in xz-directions
fprintf(f,'LSEL,S,LOC,Z, %e, %e\n',-negTol,negTol); % select all lines in the given location
fprintf(f,'LSEL,R,LOC,X, %e, %e\n',-negTol,negTol); % reselect from the current set
fprintf(f,'NSLL,S,1\n'); % select all nodes (interior to line and at keypoints) associated with the selected lines
fprintf(f,'D,ALL,UZ,0.0\n');
fprintf(f,'D,ALL,UX,0.0\n');

% supporting of nodes in xy-directions
fprintf(f,'LSEL,S,LOC,Y, %e, %e\n',-negTol,negTol); % select all lines in the given location
fprintf(f,'LSEL,R,LOC,X, %e, %e\n',-negTol,negTol); % reselect from the current set
fprintf(f,'NSLL,S,1\n'); % select all nodes (interior to line and at keypoints) associated with the selected lines
fprintf(f,'D,ALL,UY,0.0\n');
fprintf(f,'D,ALL,UX,0.0\n');

% supporting of nodes in z-direction
fprintf(f,'LSEL,S,LOC,Z, %e, %e\n',-negTol,negTol); % select all lines in the given location
fprintf(f,'LSEL,R,LOC,X, %e, %e\n',sampleDim(1)-negTol,sampleDim(1)+negTol); % reselect from the current set
fprintf(f,'NSLL,S,1\n'); % select all nodes (interior to line and at keypoints) associated with the selected lines
fprintf(f,'D,ALL,UZ,0.0\n');
fprintf(f,'LSEL,S,LOC,Z, %e, %e\n',-negTol,negTol); % select all lines in the given location
fprintf(f,'LSEL,R,LOC,Y, %e, %e\n',sampleDim(2)-negTol,sampleDim(2)+negTol); % reselect from the current set
fprintf(f,'NSLL,S,1\n'); % select all nodes (interior to line and at keypoints) associated with the selected lines
fprintf(f,'D,ALL,UZ,0.0\n');

fprintf(f,'NUMCMP,ALL\n'); % compress all numbers (renumbers the items and deletes unused ones)
fprintf(f,'GPLOT\n\n');
%% OUTPUT

fprintf(f,'!----------------------------------------------------------------------\n');
fprintf(f,'!LIST OF REQUIRED ITEMS\n');
fprintf(f,'!----------------------------------------------------------------------\n\n');

fprintf(f,'FINISH\n');
fprintf(f,'/PREP7\n');
fprintf(f,'NSEL,ALL\n');
fprintf(f,'/output,nlist.dat\n');
fprintf(f,'nlist,all, , ,xyz,node,node,node\n');
fprintf(f,'/output\n');
fprintf(f,'ESEL,ALL\n');
fprintf(f,'/output,elist.dat\n');
fprintf(f,'elist\n');
fprintf(f,'/output\n');
fprintf(f,'/output,etlist.dat\n');
fprintf(f,'etlist\n');
fprintf(f,'/output\n');
fprintf(f,'/output,dlist.dat\n');
fprintf(f,'dlist\n');
fprintf(f,'/output\n');
fprintf(f,'/output,flist.dat\n');
fprintf(f,'flist\n');
fprintf(f,'/output\n');
fprintf(f,'/output,celist.dat\n');
fprintf(f,'celist\n');
fprintf(f,'/output\n');
fprintf(f,'finish\n');

fclose(f);
display('ANSYS input file printing finished.')

if figureOut
    ax = get(p,'Parent');
    set(ax,'XLim', [0 cubeSizeRVE], 'YLim', [0 cubeSizeRVE], 'YLim', [0 cubeSizeRVE], 'PlotBoxAspectRatio', [1 1 1])
    set(ax, 'Position',[0.1 0.1 0.8 0.8])
    set(ax, 'XTick',[],'YTick',[],'ZTick',[])
    box on;
    view(26, 26);
    axis([0 cubeSizeRVE 0 cubeSizeRVE 0 cubeSizeRVE]);
    axis tight
    axis equal
    lightHandles = light;
    set(lightHandles, 'Position', [cubeSizeRVE*2.6, cubeSizeRVE*0.5, cubeSizeRVE*1.1], 'Style', 'local');
    lighting gouraud
    
    display('Saving figure...')
    pngOutput = ['RVE_',num2str(size(particlesOrientations,1)),'_crystals_c_',num2str(cAgg),'.png'];
    epsOutput = ['RVE_',num2str(size(particlesOrientations,1)),'_crystals_c_',num2str(cAgg),'.eps'];
    eval(['print -dpng outputMicrostructure/figs/',pngOutput])
    eval(['print -depsc2 outputMicrostructure/figs/',epsOutput])
end

toc
display('---------------------------------')